@@ -28,7 +28,9 @@ mod net {
     pub struct IncomingStreams {
         /// A state object, stored in an Option so we can take ownership of it
         /// while poll is being called.
-        // XXXX I hate using this trick.
+        // TODO(nickm): I hate using this trick.  At some point in the
+        // future, once Rust has nice support for async traits, maybe
+        // we can refactor it.
         state: Option<IncomingStreamsState>,
     }
     /// The result type returned by [`take_and_poll`].
@@ -119,7 +121,6 @@ mod tls {
 
     use std::convert::TryFrom;
     use std::io::{Error as IoError, Result as IoResult};
-    use std::net::SocketAddr;
 
     /// The TLS-over-TCP type returned by this module.
     #[allow(unreachable_pub)] // not actually unreachable; depends on features
@@ -140,16 +141,14 @@ mod tls {
     }
 
     #[async_trait]
-    impl crate::traits::TlsConnector for TlsConnector {
+    impl crate::traits::TlsConnector<TcpStream> for TlsConnector {
         type Conn = TlsStream;
 
-        async fn connect_unvalidated(
+        async fn negotiate_unvalidated(
             &self,
-            addr: &SocketAddr,
+            stream: TcpStream,
             hostname: &str,
         ) -> IoResult<Self::Conn> {
-            let stream = TcpStream::connect(addr).await?;
-
             let conn = self
                 .connector
                 .connect(hostname, stream)
@@ -181,6 +180,7 @@ mod tls {
 
 // ==============================
 
+use async_std_crate::net::TcpStream;
 use futures::{Future, FutureExt};
 use std::pin::Pin;
 use std::time::Duration;
@@ -205,7 +205,7 @@ impl SpawnBlocking for async_executors::AsyncStd {
     }
 }
 
-impl TlsProvider for async_executors::AsyncStd {
+impl TlsProvider<TcpStream> for async_executors::AsyncStd {
     type TlsStream = tls::TlsStream;
     type Connector = tls::TlsConnector;
 
